!----------------------------------------------------------------------------------------------
! Ordinal Number functions take a number and return it worded in "Ordinal" form
!
!   Example: 151 is One Hundred Fifty-First
!   This is different from the Cardinal form of: One Hundred Fifty-One
!   This was really meant for short numbers, e.g. the day number of a race meet like "Seventy-Second Racing Day"
!   It has been extended to allow for large numbers (< 1 billion) e,g, 1,100,155=One Million One Hundred Thousand One Hundred Fifty-Fifth
!   It could be easily extended to do Billions but I doubt it's needed
!
!   THis code could be easily converted to return Cardinal numbers, I'll leave that to you
!
! By Carl Barnes, www.carlbarnes.com
! See my website for a fe wother things and my utilites
!   Clarion Assistant - A swiss army knife of tools for the Clarion Developer
!   Clarion Source Search - Your source is a KB into your App, CSS is designed for searching Clarion Source an tapping that KB
!
!----------------------------------------------------------------------------------------------

    PROGRAM
    include 'equates.clw'
    include 'keycodes.clw'
    MAP
Num2OrdinalWords        PROCEDURE(LONG Number),STRING  !Numbers < 1 billion
SmallNum2OrdinalWords   PROCEDURE(LONG Number),STRING  !Numbers up to 9999
    .
                            
SingleNum   LONG(151)
Range1      LONG(1)
Range2      LONG(200)
RNdx        LONG,AUTO

TestQ   QUEUE
Num         LONG
SText        STRING(128)
LText        STRING(256)     !about 128 bytes cover it e.g. Seven Hundred Seventy-Seven Million Three Hundred Seventy-Three Thousand Three Hundred Seventy-Seventh  (102 bytes)
        END
MaxSize     LONG
MaxLText    STRING(256)


Window WINDOW('Test Ordinal Number String'),AT(,,395,118),FONT('MS Sans Serif',8,,),IMM,ICON(ICON:Thumbnail), |
         SYSTEM,GRAY,RESIZE
       PROMPT('&Number'),AT(2,2),USE(?Prompt1)
       SPIN(@n12),AT(33,2,81,10),USE(SingleNum),HVSCROLL,FONT(,,,FONT:bold)
       PROMPT('&Range'),AT(130,3),USE(?Prompt2)
       SPIN(@n12),AT(156,2,63,10),USE(Range1),FONT(,,,FONT:bold)
       STRING('to'),AT(223,3),USE(?String1)
       SPIN(@n12),AT(233,2,63,10),USE(Range2),FONT(,,,FONT:bold)
       BUTTON('&Generate'),AT(300,1,35,12),USE(?Generate),TIP('Max length found will be on clipboard<13,10>and highlighted in the list')
       BUTTON('&Empty'),AT(348,1,35,12),USE(?Empty),TIP('Empty the Queue')
       LIST,AT(1,19),USE(?List:TestQ),FULL,VSCROLL,FORMAT('53L(1)|M~Number~@n12@141L(1)|M~Small Ordinal~@s128@20L(1)~Oridinal~'), |
           FROM(TestQ)
     END
          
    CODE
    OPEN(Window)
    POST(EVENT:Accepted,?SingleNum)
    ACCEPT
        CASE EVENT()
        OF EVENT:Accepted
        OROF EVENT:NewSelection

                CASE FIELD()
                OF  ?SingleNum
                     TestQ.Num  = SingleNum
                     TestQ.LText = Num2OrdinalWords(SingleNum)
                     TestQ.SText = SmallNum2OrdinalWords(SingleNum)
                     ADD(TestQ,1) ; ?List:TestQ{PROP:Selected} = 1
                     SELECT(?SingleNum)
                     DISPLAY
                OF ?Generate
                    FREE(TestQ)
                    DISPLAY
                    MaxSize=0 ; MaxLText = ''
                    LOOP RNdx = Range1 TO Range2 BY 1
                         TestQ.Num  = RNdx
                         TestQ.LText = Num2OrdinalWords(RNdx)
                         TestQ.SText = SmallNum2OrdinalWords(RNdx)
                         ADD(TestQ)
                         if maxSize < len(clip(TestQ.LText))
                            maxSize = len(clip(TestQ.LText))
                            MaxLText = TestQ.LText
                            ?List:TestQ{PROP:Selected} = records(TestQ)
                            display
                         end
                    END
                    setclipboard(CLIP(MaxLText) & '  (' & maxSize & ' bytes)' )
                    DISPLAY

                OF ?Range1
                    IF Range2 < Range1
                       Range2 =Range1
                       display
                    END

                OF ?List:TestQ
                    IF KEYCODE()=MouseLeft2
                        GET(testQ,choice(?List:TestQ))
                        SETCLIPBOARD(clip(TestQ.LText))
                    END
                OF ?Empty
                    FREE(TestQ)
                    DISPLAY
                END

        END !CASE EVENT
    END


Num2OrdinalWords     PROCEDURE(LONG Number)!,STRING

Ords    STRING('First      Second     Third      Fourth     Fifth      Sixth      Seventh    ' & |
               'Eighth     Ninth      Tenth      Eleventh   Twelfth    Thirteenth Fourteenth ' & |
               'Fifteenth  Sixteenth  SeventeenthEighteenth Nineteenth Twentieth  Thirtieth  ' & |
               'Fortieth   Fiftieth   Sixtieth   Seventieth Eightieth  Ninetieth  ')
Ordinal STRING(11),DIM(27),OVER(Ords)

Cards   STRING('One      Two      Three    Four     Five     Six      Seven    Eight    Nine     ' & |
               'Ten      Eleven   Twelve   Thirteen Fourteen Fifteen  Sixteen  SeventeenEighteen ' & |
               'Nineteen Twenty   Thirty   Forty    Fifty    Sixty    Seventy  Eighty   Ninety   ')
Cardinal   STRING(9),DIM(27),OVER(Cards)

NumString   STRING(12),AUTO     !Make this 3 larger than the largest number
Num3Idx     LONG,AUTO
Magnitude   LONG,AUTO
Hundreds    LONG,AUTO
Tens        LONG,AUTO
Ones        LONG,AUTO
Remainder   LONG,AUTO
MoreLater   LONG,AUTO           !If there is a Remainder or we are > 999 there is more coming so no Ordinals
RetNum      STRING(256)
    CODE
    IF Number > 999999999 THEN
       RETURN( CLIP(LEFT(FORMAT(Number,@n13))) & CHOOSE(Number % 10,'st','nd','rd','th') )
    END


    NumString = FORMAT(Number,@n09)
    LOOP Magnitude = 1 TO 3                         !Take number in chunks of Three
         Num3Idx = Magnitude * 3 - 2
         IF NumString[Num3Idx : Num3Idx + 2] > 0
            Hundreds  = NumString[Num3Idx]
            Tens      = NumString[Num3Idx + 1]
            Ones      = NumString[Num3Idx + 2]
            Remainder = NumString[Num3Idx + 3 : SIZE(NumString)]
            MoreLater = CHOOSE(Remainder>0 OR Magnitude<3,1,0)
            DO ThreeDigitsRtn
            IF Remainder = 0                                   !Remainder zero, time for Ordinal suffix
               RetNum=CLIP(RetNum) &' '& CHOOSE(Magnitude,'Millionth','Thousandth','')
            ELSE                                               !If more to do then Cardinal
               RetNum=CLIP(RetNum) &' '& CHOOSE(Magnitude,'Million','Thousand','')
            END
         END !IF NumString > 0
    END
    IF ~RetNum THEN RetNum='Zeroth'.
    RETURN(CLIP(LEFT(RetNum)))

ThreeDigitsRtn  ROUTINE

    IF Hundreds
       RetNum=CLIP(RetNum) &' '& CLIP(Cardinal[Hundreds]) & CHOOSE(Tens+Ones+MoreLater=0,' Hundredth',' Hundred')
    END
    CASE Tens
    OF 0 ; IF Ones THEN RetNum=CLIP(RetNum) &' '& CHOOSE(~MoreLater,Ordinal[Ones],Cardinal[Ones]).
    OF 1 ; RetNum=CLIP(RetNum) &' '& CHOOSE(~MoreLater,Ordinal[10+Ones],Cardinal[10+Ones])
    ELSE                                                                !20-99
           IF Ones                                                      !21,22,23..
              RetNum=CLIP(RetNum) &' '& CLIP(Cardinal[18+Tens]) & '-' & |
                                          CHOOSE(~MoreLater,Ordinal[Ones],Cardinal[Ones])
           ELSE                                                        !20,30,40,50
              RetNum=CLIP(RetNum) &' '& CHOOSE(~MoreLater,Ordinal[18+Tens],Cardinal[18+Tens])
           END
    END
    EXIT

    !----------------------------------------------------------------------------------------------
    !--SMALL this version can be much simpler since it only allows up to 9999
    !   It also does Teens for 1100 to 1999, e.g.  Thriteen Hundredth

SmallNum2OrdinalWords     PROCEDURE(LONG Number)!,STRING   !Limit of 9999

Ords    STRING('First      Second     Third      Fourth     Fifth      Sixth      Seventh    ' & |
               'Eighth     Ninth      Tenth      Eleventh   Twelfth    Thirteenth Fourteenth ' & |
               'Fifteenth  Sixteenth  SeventeenthEighteenth Nineteenth Twentieth  Thirtieth  ' & |
               'Fortieth   Fiftieth   Sixtieth   Seventieth Eightieth  Ninetieth  ')
Ordinal STRING(11),DIM(27),OVER(Ords)

Cards   STRING('One      Two      Three    Four     Five     Six      Seven    Eight    Nine     ' & |
               'Ten      Eleven   Twelve   Thirteen Fourteen Fifteen  Sixteen  SeventeenEighteen ' & |
               'Nineteen Twenty   Thirty   Forty    Fifty    Sixty    Seventy  Eighty   Ninety   ')
Cardinal   STRING(9),DIM(27),OVER(Cards)

NumString   STRING(4),AUTO     
Thousands   LONG,AUTO
Hundreds    LONG,AUTO
Tens        LONG,AUTO
Ones        LONG,AUTO
RetNum      STRING(256)
    CODE
    IF Number > 9999 THEN
       RETURN( CLIP(LEFT(FORMAT(Number,@n13))) & CHOOSE(Number % 10,'st','nd','rd','th') )
    END

    ASSERT(Number<10000,'Max is 9999')
    NumString = FORMAT(Number,@n04)
    Hundreds  = NumString[1:2]
    Tens      = NumString[3]  
    Ones      = NumString[4]  


    IF Hundreds = 10 OR Hundreds > 19
       Thousands = Hundreds / 10
       Hundreds %= 10
       RetNum=CLIP(RetNum) &' '& CLIP(Cardinal[Thousands]) & CHOOSE(Hundreds+Tens+Ones=0,' Thousandth',' Thousand')
    END
    IF Hundreds
       RetNum=CLIP(RetNum) &' '& CLIP(Cardinal[Hundreds]) & CHOOSE(Tens+Ones=0,' Hundredth',' Hundred')
    END
    CASE Tens
    OF 0          
        IF Ones THEN RetNum=CLIP(RetNum) &' '& Ordinal[Ones].
    OF 1          
        RetNum=CLIP(RetNum) &' '& Ordinal[10+Ones]                !Teens?
    ELSE
        IF Ones
           RetNum=CLIP(RetNum) &' '& CLIP(Cardinal[18+Tens]) & '-' & Ordinal[Ones]
        ELSE
           RetNum=CLIP(RetNum) &' '& Ordinal[18+Tens]
        END

    END
    IF ~RetNum THEN RetNum='Zeroth'.
    RETURN(LEFT(RetNum))

